use std::ops::Deref;
use std::sync::atomic::{AtomicUsize, Ordering};

use crate::logger::Logger;



/// The amount of references that have currently been created.
static REF_COUNT: AtomicUsize = AtomicUsize::new(0);



/// This is an access guard for the Logger.
/// It helps keep track of REF_COUNTS. Since there can be
/// only one GLOBAL_LOGGER that needs guarded, this is done
/// with static variables.
pub struct Guard
{
   logger: &'static dyn Logger
}


impl Guard
{
   /// Create a new reference counted Guard.
   pub fn new(global_logger: *const dyn Logger) -> Guard
   {
      REF_COUNT.fetch_add(1, Ordering::SeqCst);
      unsafe
      {
         Guard
         {
            logger: &*global_logger
         }
      }
   }

   /// Do a spin lock while waiting for the desired REF_COUNT to be hit.
   /// Normally, this is waiting for REF_COUNT to be zero.
   pub fn spin_until(desired_refcount: usize)
   {
      while REF_COUNT.load(Ordering::SeqCst) != desired_refcount
      {
         // It would be nice to sleep this thread here,
         // but I'm not sure how to do that without the STD.
      }
   }
}

impl Drop for Guard
{
   fn drop(&mut self)
   {
      REF_COUNT.fetch_sub(1, Ordering::SeqCst);
   }
}

impl Deref for Guard
{
   type Target = dyn Logger;

   fn deref(&self) -> &(dyn Logger + 'static)
   {
      self.logger
   }
}
